/*
           LTR41.
            ,    
           :
    ltr41_read_port  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd/LtrServer (     )
     :    ,       

        ,  ,  
      ,       .

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr41api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .    ,   
           ( Consolas).
*/


#include <stdio.h>
#include <stdlib.h>
#ifdef _WIN32
#include <locale.h>
#endif

#include "ltr/include/ltr41api.h"


typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;

/*    .   ,  
 *   :
 * 1  -   ( 1  16)
 * 2  -   
 * 3  - ip-  */
static int f_get_params(int argc, char** argv, t_open_param* par) {
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;

    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3) {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++) {
            if ((a[i]<0) || (a[i] > 255)) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err) {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}
/*------------------------------------------------------------------------------------------------*/
int main(int argc, char *argv[]) {
    INT err = LTR_OK;
    TLTR41 hltr41;
    t_open_param par;

#ifdef _WIN32
    /*         Windows*/
    setlocale(LC_ALL, "");
#endif

    err = f_get_params(argc, argv, &par);

    if (err == LTR_OK) {
        LTR41_Init(&hltr41);
        /*     */
        err = LTR41_Open(&hltr41, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
        if (err != LTR_OK) {
            fprintf(stderr, "     .  %d (%s)\n",
                    err, LTR41_GetErrorString(err));
        } else {
            /*     */
            printf("  :\n");
            printf("  :        %s\n", hltr41.ModuleInfo.Name);
            printf("   :  %s\n", hltr41.ModuleInfo.Serial);
            printf("   : %s\n", hltr41.ModuleInfo.FirmwareVersion);
            printf("   :   %s\n", hltr41.ModuleInfo.FirmwareDate);


            /*   (  -) */
            err = LTR41_Config(&hltr41);
            if (err != LTR_OK) {
                fprintf(stderr, "   .  %d (%s)\n",
                        err, LTR41_GetErrorString(err));
            }
        }

        if (err == LTR_OK) {
            WORD wrd;
            err = LTR41_ReadPort(&hltr41, &wrd);
            if (err != LTR_OK) {
                fprintf(stderr, "     .  %d (%s)\n",
                        err, LTR41_GetErrorString(err));
            } else {
                WORD msk;
                printf("  : ");
                for (msk = 0x8000; msk != 0; msk>>=1)
                    printf("%d ", wrd & msk ? 1 : 0);
                printf("\n");
            }
        }

        if (LTR41_IsOpened(&hltr41) == LTR_OK)
            LTR41_Close(&hltr41);
    }


    return err;
}
