/*
          LTR41    .
            ,    
           :
    ltr41_stream_recv  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd/LtrServer (     )
     :    ,       

         ,   
      (  ),         ,
        .

           Windows   CTRL+C  Linux.

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr41api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .    ,   
           ( Consolas).
*/


#include <stdio.h>
#include <stdlib.h>
#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#else
#include <signal.h>
#include <unistd.h>
#endif

#include "ltr/include/ltr41api.h"

/*   ( ) */
#define STREAM_FREQ     100000
/*     */
#define RECV_BLOCK_SIZE 10000


/*      */
static int f_out = 0;

#ifndef _WIN32
/*     Linux */
static void f_abort_handler(int sig) {
    f_out = 1;
}
#endif

typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;

/*    .   ,  
 *   :
 * 1  -   ( 1  16)
 * 2  -   
 * 3  - ip-  */
static int f_get_params(int argc, char** argv, t_open_param* par) {
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;

    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3) {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++) {
            if ((a[i]<0) || (a[i] > 255)) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err) {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}
/*------------------------------------------------------------------------------------------------*/
int main(int argc, char *argv[]) {
    INT err = LTR_OK;
    TLTR41 hltr41;
    t_open_param par;

#ifndef _WIN32
    struct sigaction sa;
    /*   Linux      ,
           */
    sa.sa_handler = f_abort_handler;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGABRT, &sa, NULL);
#endif
#ifdef _WIN32
    /*         Windows*/
    setlocale(LC_ALL, "");
#endif

    err = f_get_params(argc, argv, &par);

    if (err == LTR_OK) {
        LTR41_Init(&hltr41);
        /*     */
        err = LTR41_Open(&hltr41, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
        if (err != LTR_OK) {
            fprintf(stderr, "     .  %d (%s)\n",
                    err, LTR41_GetErrorString(err));
        } else {
            /*     */
            printf("  :\n");
            printf("  :        %s\n", hltr41.ModuleInfo.Name);
            printf("   :  %s\n", hltr41.ModuleInfo.Serial);
            printf("   : %s\n", hltr41.ModuleInfo.FirmwareVersion);
            printf("   :   %s\n", hltr41.ModuleInfo.FirmwareDate);

            /*     */
            hltr41.StreamReadRate = STREAM_FREQ;
            err = LTR41_Config(&hltr41);
            if (err != LTR_OK) {
                fprintf(stderr, "   .  %d (%s)\n",
                        err, LTR41_GetErrorString(err));
            }
        }

        if (err == LTR_OK) {
            /*     + 4c */
            DWORD tout = 4000 + (DWORD)(RECV_BLOCK_SIZE*1000/hltr41.StreamReadRate);
            /*     .  LTR */
            DWORD *rbuf = (DWORD*)malloc(RECV_BLOCK_SIZE*sizeof(rbuf[0]));
            /*    ( ) */
            WORD *wrds = (WORD*)malloc(RECV_BLOCK_SIZE*sizeof(wrds[0]));
            /*    */
            DWORD *tmark = (DWORD*)malloc(RECV_BLOCK_SIZE*sizeof(tmark[0]));
            DWORD block = 0;
            WORD prev_word = 0;
            DWORD prev_tmark = 0;

            if ((rbuf == NULL) || (wrds == NULL) || (tmark == NULL)) {
                err = LTR_ERROR_MEMORY_ALLOC;
                fprintf(stderr, "  !\n");
            }

            if (err == LTR_OK) {
                /*    */
                err = LTR41_StartStreamRead(&hltr41);
                if (err != LTR_OK) {
                    fprintf(stderr, "   .  %d (%s)\n",
                            err, LTR41_GetErrorString(err));
                } else {
                    INT stop_err;
                    printf("  .    %s\n",
#ifdef _WIN32
                           " "
#else
                           "CTRL+C"
#endif
                           );
                    fflush(stdout);

                    printf("    , :  16 - 1                       \n");

                    while ((err == LTR_OK) && !f_out) {
                        INT read_size = LTR41_Recv(&hltr41, rbuf, tmark, RECV_BLOCK_SIZE, tout);
                        if (read_size < 0) {
                            /*   0 -  */
                            err = read_size;
                            fprintf(stderr, "  .  %d (%s)\n",
                                    err, LTR41_GetErrorString(err));
                        } else if (read_size != RECV_BLOCK_SIZE) {
                            /* ..      , 
                             *     ( )
                             *   */
                            err = LTR_ERROR_RECV_INSUFFICIENT_DATA;
                            fprintf(stderr, "  .  %d,  %d\n",
                                    RECV_BLOCK_SIZE, read_size);
                        } else {
                            DWORD proc_size = read_size;
                            err = LTR41_ProcessData(&hltr41, rbuf, wrds, &proc_size);
                            if (err != LTR_OK) {
                                fprintf(stderr, "  .  %d (%s)\n",
                                        err, LTR41_GetErrorString(err));
                            } else {
                                DWORD i;
                                for (i=0; i < proc_size; i++) {
                                    int out = 0;
                                    /*      
                                     *   ,   
                                     *   */
                                    if (prev_tmark != tmark[i]) {
                                        prev_tmark = tmark[i];
                                        out = 1;
                                    }
                                    if (prev_word != wrds[i]) {
                                        prev_word = wrds[i];
                                        out = 1;
                                    }

                                    if (out) {
                                        WORD sec = prev_tmark & 0xFFFF;
                                        WORD start = (prev_tmark >> 16) & 0xFFFF;
                                        WORD msk;

                                        printf("%8d, %7d: ", block, i);
                                        for (msk = 0x8000; msk != 0; msk>>=1)
                                            printf("%d ", prev_word & msk ? 1 : 0);
                                        printf("%5d    %5d\n", start, sec);
                                    }
                                }

                                block++;
                            }
                        }

#ifdef _WIN32
                        /*      */
                        if (err==LTR_OK) {
                            if (_kbhit())
                                f_out = 1;
                        }
#endif
                    }

                    /*    */
                    stop_err = LTR41_StopStreamRead(&hltr41);
                    if (stop_err != LTR_OK) {
                        fprintf(stderr, "   .  %d (%s)\n",
                                stop_err, LTR41_GetErrorString(stop_err));
                        if (err == LTR_OK)
                            err = stop_err;
                    } else {
                        printf("   !\n");
                    }
                }
            }


            free(rbuf);
            free(wrds);
            free(tmark);
        }

        if (LTR41_IsOpened(&hltr41) == LTR_OK)
            LTR41_Close(&hltr41);
    }


    return err;
}
